from edupils import constantes, desenho
import asyncio


class Objeto:
    def __init__(
        self,
        nome,
        posicao_inicial,
        altura,
        funcao_movimento=None,
        forma="quadrado",
        cor="azul",
        origem_em_metros=(10, 1),
        pixels_por_metro=25,
    ):
        self.nome = nome
        self.posicao = posicao_inicial
        self.altura = altura
        self.funcao_movimento = funcao_movimento or (lambda t: self.posicao)
        self.forma = forma
        self.cor = cor
        self.origem_metros = origem_em_metros
        self.pixels_por_metro = pixels_por_metro
        self.desenhar()

    def desenhar(self, camada=constantes.NOME_PAINEL_FRENTE):
        x = (self.posicao + self.origem_metros[0]) * self.pixels_por_metro
        y = (self.altura + self.origem_metros[1] - 1) * self.pixels_por_metro
        if self.forma == "quadrado":
            desenho.desenhar_retangulo(
                x, y, 20, 20, camada, cor_preenchimento=self.cor
            )
        elif self.forma == "triangulo":
            desenho.desenhar_triangulo(
                x+10, y+12, 14, self.cor, camada, angulo=270, proporcao_base=1.61
            )
        elif self.forma == "circulo":
            desenho.desenhar_arco(
                x+10, y+10, 10, 0, 360, camada, cor_preenchimento=self.cor, cor_contorno=self.cor, largura_contorno=1
            )



class Animacao:
    def __init__(
        self,
        tempo=10,
        frames_por_segundo=10,
        distancia_em_metros=20,
        origem_em_metros=(10, 1),
        pixels_por_metro=25,
        deixar_rastro=False,
    ):
        self.tempo = tempo
        self.frames_por_segundo = frames_por_segundo
        self.distancia_em_metros = distancia_em_metros
        self.origem_em_metros = origem_em_metros
        self.pixels_por_metro = pixels_por_metro
        self.deixar_rastro = deixar_rastro
        self.objetos = {}
        self.desenhar_eixo_x()

    def adicionar_objeto(
        self,
        nome,
        posicao_inicial,
        funcao_movimento,
        forma="quadrado",
        cor="azul"
    ):
        self.objetos[nome] = Objeto(
            nome,
            posicao_inicial,
            0,
            funcao_movimento,
            forma=forma,
            cor=cor,
            origem_em_metros=self.origem_em_metros,
            pixels_por_metro=self.pixels_por_metro,
        )

    def desenhar_eixo_x(self, camada=constantes.NOME_PAINEL_AUXILIAR):
        desenho.apagar_painel(camada)
        inicio_y = self.origem_em_metros[1] * self.pixels_por_metro
        fim_y = inicio_y
        inicio_x = (
            (self.origem_em_metros[0] - self.distancia_em_metros)
            * self.pixels_por_metro
            * 2
        )
        fim_x = (
            (self.origem_em_metros[0] + self.distancia_em_metros)
            * self.pixels_por_metro
            * 2
        )
        desenho.desenhar_linha(inicio_x, inicio_y, fim_x, fim_y, id_canvas=camada)

        for i in range(-2 * self.distancia_em_metros, 2 * self.distancia_em_metros + 1):
            x = (i + self.origem_em_metros[0]) * self.pixels_por_metro
            desenho.escrever_texto(
                str(i), x + 2, inicio_y + 12, id_canvas=camada, tamanho=10
            )
            desenho.desenhar_linha(x, inicio_y, x, inicio_y + 5, id_canvas=camada)

    def desenhar_tempo(self, t, camada=constantes.NOME_PAINEL_FRENTE):
        desenho.escrever_texto(
            f"t = {t:.1f} s",
            400,
            100,
            id_canvas=camada,
            tamanho=12,
            cor="black",
        )

    def desenhar_quadro(self, t, camada=constantes.NOME_PAINEL_FRENTE):
        desenho.apagar_painel(camada)
        self.desenhar_tempo(t, camada=camada)
        for obj in self.objetos.values():
            obj.posicao = obj.funcao_movimento(t)
            obj.desenhar(camada=camada)

    def desenhar_rastro(self, t, camada=constantes.NOME_PAINEL_FUNDO):
        for obj in self.objetos.values():
            obj.posicao = obj.funcao_movimento(t)
            obj.desenhar(camada=camada)
        desenho.clarear_com_marca_dagua(camada, .6)

    def apagar_tudo(self):
        for camada in [constantes.NOME_PAINEL_FUNDO, constantes.NOME_PAINEL_FRENTE]:
            desenho.apagar_painel(camada)

    async def animar(self):
        self.apagar_tudo()
        max_steps = int(self.tempo * self.frames_por_segundo)
        dt = 1 / self.frames_por_segundo

        for step in range(max_steps):
            t = step * dt
            
            self.desenhar_quadro(t)
            if self.deixar_rastro and (t % 1 < .001):
                self.desenhar_rastro(t, camada=constantes.NOME_PAINEL_FUNDO)
            
            await asyncio.sleep(dt)

class Animacao2D(Animacao):
    pass



from grandezas import Forca
from vetor import Vetor
from abc import abstractmethod

class ForcaVetorial(Vetor):

    def __init__(self, x, y):
        super().__init__(Forca(x), Forca(y))
    
    @abstractmethod
    def atualizar(self):
        pass

class ForcaAtrito(ForcaVetorial):
    def __init__(self, x, y, coeficiente_atrito=0.1):
        super().__init__(x, y)
    
    def atualizar(self, forca_normal, velocidade):
        pass

class ForcaElastica(ForcaVetorial):
    def __init__(self, x, y, constante_elastica=1, distancia_repouso=1):
        super().__init__(x, y)
        self.constante_elastica = constante_elastica
        self.distancia_repouso = distancia_repouso
    
    def atualizar(self, posicao_atual, posicao_outra_extremidade):
        posicao_relativa = posicao_outra_extremidade - posicao_atual
        deformacao = posicao_relativa.norma() - self.distancia_repouso
        self.x, self.y = (posicao_relativa.normalizar() * self.constante_elastica * deformacao).aplicar(Forca)

class ForcaGravitacao(ForcaVetorial):
    def __init__(self, x, y, constante_gravitacao=1):
        super().__init__(x, y)
        self.constante_gravitacao = constante_gravitacao
    
    def atualizar(self, massa1, massa2, posicao1, posicao2):
        posicao_relativa = posicao2 - posicao1
        self.x, self.y = (posicao_relativa.normalizar() * self.constante_gravitacao * massa1 * massa2 / posicao_relativa.norma()**2).aplicar(Forca)

class ForcaEletromagnetica(ForcaVetorial):
    def __init__(self, x, y, constante_eletromagnetica=1):
        super().__init__(x, y)
        self.constante_eletromagnetica = constante_eletromagnetica
    
    def atualizar(self, carga1, carga2, posicao1, posicao2, velocidade1, velocidade2):
        posicao_relativa = posicao2 - posicao1
        velocidade_relativa = velocidade2 - velocidade1
        self.x, self.y = (posicao_relativa.normalizar() * self.constante_eletromagnetica * carga1 * carga2 / posicao_relativa.norma()**2).aplicar(Forca) + (velocidade_relativa.normalizar() * self.constante_eletromagnetica * carga1 * carga2 / posicao_relativa.norma()**2).aplicar(Forca)


class ForcaArrasto(ForcaVetorial):
    def __init__(self, x, y, coeficiente_arrasto=1):
        super().__init__(x, y)
        self.coeficiente_arrasto = coeficiente_arrasto
    
    def atualizar(self, velocidade):
        self.x, self.y = (velocidade.normalizar() * self.coeficiente_arrasto * velocidade.norma()**2).aplicar(Forca)


if __name__ == "__main__":
    from grandezas import Posicao

    forca = ForcaElastica(1, 1)
    print(forca)  # Vetor(1.0 N, 1.0 N)

    forca.atualizar(Vetor(Posicao(0), Posicao(0)), Vetor(Posicao(1), Posicao(1)))

    print(forca)


class GrandezaFisica(float):
    unidade = ""

    def __new__(cls, valor):
        return float.__new__(cls, valor)

    def __repr__(self):
        return f"{super().__repr__()} {self.unidade}"

    def __add__(self, outro):
        if isinstance(outro, GrandezaFisica) and self.unidade == outro.unidade:
            return self.__class__(super().__add__(outro))
        else:
            raise TypeError(f"Não é possível somar {self.unidade} com {outro.unidade}")
        
    def __sub__(self, outro):
        if isinstance(outro, GrandezaFisica) and self.unidade == outro.unidade:
            return self.__class__(super().__sub__(outro))
        else:
            raise TypeError(f"Não é possível subtrair {self.unidade} com {outro.unidade}")
        
    def __mul__(self, outro):
        if isinstance(outro, (int, float)) and not isinstance(outro, GrandezaFisica):
            return self.__class__(super().__mul__(outro))
        else:
            raise TypeError(f"Não é possível multiplicar {self.unidade} com {outro.unidade}")
    
    def __rmul__(self, outro):
        if isinstance(outro, (int, float)) and not isinstance(outro, GrandezaFisica):
            return self.__class__(super().__mul__(outro))
        else:
            raise TypeError(f"Não é possível multiplicar {self.unidade} com {outro.unidade}")

    def __truediv__(self, outro):
        if isinstance(outro, (int, float)) and not isinstance(outro, GrandezaFisica):
            return self.__class__(super().__truediv__(outro))
        else:
            raise TypeError(f"Não é possível dividir {self.unidade} por {outro.unidade}")
        
    

class Tempo(GrandezaFisica):
    unidade = "s"

class Posicao(GrandezaFisica):
    unidade = "m"

    def __truediv__(self, outro):
        if isinstance(outro, Tempo):
            return Velocidade(self / outro)
        else:
            return super().__truediv__(outro)


class Velocidade(GrandezaFisica):
    unidade = "m/s"

    def __mul__(self, outro):
        if isinstance(outro, Tempo):
            return Posicao(float(self) * float(outro))
        else:
            return super().__mul__(outro)

    def __rmul__(self, outro):
        if isinstance(outro, Tempo):
            return Posicao(float(self) * float(outro))
        else:
            return super().__rmul__(outro)
        
    def __truediv__(self, outro):
        if isinstance(outro, Tempo):
            return Aceleracao(float(self) / float(outro))
        else:
            return super().__truediv__(outro)


class Aceleracao(GrandezaFisica):
    unidade = "m/s^2"

    def __mul__(self, outro):
        if isinstance(outro, Tempo):
            return Velocidade(float(self) * float(outro))
        else:
            return super().__mul__(outro)
        
    def __rmul__(self, outro):
        if isinstance(outro, Tempo):
            return Velocidade(float(self) * float(outro))
        else:
            return super().__rmul__(outro)
        
    def __truediv__(self, outro):
        if isinstance(outro, Tempo):
            raise TypeError(f"A taxa de variação da aceleração em ({self.unidade}) no tempo em ({outro.unidade}) é uma grandeza física que não usaremos nesta atividade.")
        else:
            return super().__truediv__(outro)

class Massa(GrandezaFisica):
    unidade = "kg"

class Forca(GrandezaFisica):
    unidade = "N"

    def __truediv__(self, outro):
        if isinstance(outro, Massa):
            return Aceleracao(float(self) / float(outro))
        else:
            return super().__truediv__(outro)

class Energia(GrandezaFisica):
    unidade = "J"

if __name__ == "__main__":
    # Somando duas grandezas do mesmo tipo
    tempo1 = Tempo(10)
    tempo2 = Tempo(5)
    print(tempo1 + tempo2)  # 15.0 s

    # Multiplicando velocidade por tempo para obter posição
    velocidade = Velocidade(2)
    tempo = Tempo(3)
    posicao = velocidade * tempo
    print(posicao)  # 6.0 m

    # Multiplicando aceleração por tempo para obter velocidade
    aceleracao = Aceleracao(9.8)
    tempo = Tempo(2)
    velocidade = aceleracao * tempo
    print(velocidade)  # 19.6 m/s


class Vetor:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def norma(self):
        return (self.x**2 + self.y**2)**0.5
    
    def normalizar(self):
        norma = self.norma()
        return self / norma if norma != 0 else Vetor(0, 0)

    def produto_escalar(self, outro):
        return self.x * outro.x + self.y * outro.y

    def projetar(self, outro):
        produto_escalar = self.produto_escalar(outro)
        norma_quadrado = outro.norma()**2
        if norma_quadrado != 0:
            fator = produto_escalar / norma_quadrado
            return Vetor(fator * outro.x, fator * outro.y)
        return Vetor(0, 0)  # Caso o vetor sobre o qual estamos projetando tenha norma 0, retorna um vetor nulo
    
    def perpendicular(self):
        return Vetor(-self.y, self.x)
    
    def aplicar(self, f):
        return Vetor(f(self.x), f(self.y))
    
    def __add__(self, outro):
        return Vetor(self.x + outro.x, self.y + outro.y)

    def __sub__(self, outro):
        return Vetor(self.x - outro.x, self.y - outro.y)
    
    def __repr__(self):
        return f"Vetor({self.x}, {self.y})"
    
    def __mul__(self, escalar):
        if isinstance(escalar, (int, float)):
            return Vetor(self.x * escalar, self.y * escalar)
        else:
            raise TypeError("O operador deve ser um número inteiro ou flutuante")

    def __rmul__(self, escalar):
        # Isso permite a multiplicação escalar tanto à esquerda quanto à direita do vetor
        return self.__mul__(escalar)
    
    def __truediv__(self, escalar):
        if isinstance(escalar, (int, float)):
            return Vetor(self.x / escalar, self.y / escalar)
        else:
            raise TypeError("O operador deve ser um número inteiro ou flutuante")

    def __iter__(self):
        yield self.x
        yield self.y

if __name__ == "__main__":
    from grandezas import Posicao, Velocidade, Tempo

    p = Vetor(
        Posicao(0),
        Posicao(-1),
    )

    v = Vetor(
        Velocidade(-1),
        Velocidade(2)
    )

    print(p + v * Tempo(2))


from js import document

def change_icon_color(class_name, color):
    # Assuming 'bi-play-circle' is the class for the icons inside the buttons you want to target
    icons = document.getElementsByClassName(class_name)
    for icon in icons:
        icon.style.color = color

# Example usage: Change the color of icons within buttons to purple
change_icon_color('bi-play-circle', 'purple')

def change_button_background(class_name, color):
    # Get all elements (buttons) with the specified class name
    buttons = document.getElementsByClassName(class_name)
    for button in buttons:
        # Change the background color of each button
        button.style.backgroundColor = color

# Change the background color of buttons with class 'cells-control-button' to purple
change_button_background('py-1', 'white')


from vetor import Vetor
import grandezas
#from edupils import desenho
from edupils import constantes
import math
import asyncio

class Simulacao:
    def __init__(
            self, 
            largura=500,
            altura=300,
            raio_max=50,
            pixels_por_metro=10,
            tempo_total=grandezas.Tempo(10),
            tempo_por_frame=grandezas.Tempo(.1),
            plotar_escala=True,
            plotar_trajetoria=False,
            plotar_velocidade=False,
            painel_frente = constantes.NOME_PAINEL_FRENTE,
            painel_fundo = constantes.NOME_PAINEL_FUNDO,
            painel_auxiliar=constantes.NOME_PAINEL_AUXILIAR,
            adicionar_gravidade=True
        ):
        self.largura = largura
        self.altura = altura
        self.velocidade_maxima = min(altura, largura) / 2
        self.raio_maximo = raio_max  # Define o tamanho de cada célula na grade
        
        self.objetos = []  # Lista para armazenar os objetos na simulação
        self.forcas = []

        self.T = tempo_total
        self.dt = tempo_por_frame

    def adicionar_objeto(self, objeto):
        """Adiciona um novo objeto à simulação e o posiciona na grade espacial."""
        assert objeto.raio <= self.raio_maximo
        self.objetos.append(objeto)
    
    def ordenar_objetos(self):
        self.objetos = sorted(self.objetos, key=lambda ob: ob.x)

    def varredura_colisoes(self):
        self.ordenar_objetos()
        colisoes = []
        for i in range(len(self.objetos)-1):
            for j in range(i+1, len(self.objetos)):
                if self.detectar_colisao_entre_objetos(self.objetos[i], self.objetos[j]):
                    colisoes.append((i, j))
                
                if ((self.objetos[i].x + self.raio_maximo) < 
                    (self.objetos[j].x - self.raio_maximo)):
                    break

    def detectar_colisao_entre_objetos(self, objeto1, objeto2):
        """Verifica se há colisão entre dois objetos."""
        dx = objeto1.x - objeto2.x
        dy = objeto1.y - objeto2.y
        distancia = (dx**2 + dy**2)**0.5
        return distancia < (objeto1.raio + objeto2.raio)


    def resolver_colisao(self, objeto1, objeto2):
        """Resolve a colisão ajustando as posições e velocidades dos objetos."""
        normal = (objeto1 - objeto2)
        distancia = normal.norma()
        normal_unitaria = normal.normalizar()
        
        deslocamento = (objeto1.raio + objeto2.raio - distancia) / 2

        tangente = normal.perpendicular()
        
        # Ajustar posições
        objeto1.posicao += normal_unitaria * deslocamento
        objeto2.posicao -= normal_unitaria * deslocamento



    def tratar_colisoes_rigidas(self):
        for objeto in self.objetos:
            if objeto.posicao.x - objeto.raio < 0:
                objeto.posicao.x = objeto.raio
                objeto.velocidade.x = -objeto.velocidade.x
            elif objeto.posicao.x + objeto.raio >  self.largura:
                objeto.posicao.x = self.largura - objeto.raio
                objeto.velocidade.x = -objeto.velocidade.x
            if objeto.posicao.y - objeto.raio < 0:
                objeto.posicao.y = objeto.raio
                objeto.velocidade.y = -objeto.velocidade.y
            elif objeto.posicao.y + objeto.raio > self.altura:
                objeto.posicao.y = self.altura - objeto.raio
                objeto.velocidade.y = -objeto.velocidade.y

    def tratar_colisoes_entre_objetos(self):
        while False:
            self.varredura_colisoes()
        
        pass

    def atualizar_forcas_externas(self):
        pass

    def atualizar_estado(self):
        """Atualiza o estado da simulação."""
        #Objeto simulação atualiza as forças externas dentro de cada objeto cadastrado dentro de sua lista de objetos (pares ação reação, molas, gravitações, magnetismos)
        
        self.atualizar_forcas_externas()

        # Cada objeto internamente atualiza seu estado
        for objeto in self.objetos:
            objeto.atualiza_estado(self.dt)

        # Checar colisões contra corpos imóveis, quiques (paredes apenas, por enquanto) e resolver o quique: Atualizar posiç~ao e atualizar velocidade
        self.tratar_colisoes_rigidas()

        # Checar colisões contra outros objetos por varredura e resolver choque elastico.
        self.tratar_colisoes_entre_objetos()

    def simular(self):
        t = 0

        while t < self.T:
            self.atualizar_estado()
            t += self.dt


    def plotar_eixos():
        pass

    def plotar_velocidades():
        pass

    def plotar_trajetoria():
        pass


class Objeto:
    def __init__(self, x, y, vx, vy, massa=1):
        self.posicao = Vetor(
            grandezas.Posicao(x), 
            grandezas.Posicao(y)
        )
    
        self.velocidade = Vetor(
            grandezas.Velocidade(vx), 
            grandezas.Velocidade(vy)
        )
        self.massa = grandezas.Massa(massa)
        self.forcas = [] # que só dependendem do objeto como gravidade, atrito, arrasto
        self.forcas_externas = {} #que dependem de outros objetos, ação e reação, como molas, magnetismos, 

    def adiciona_forca(self, forca):
        self.forcas.append(forca)
    
    def aceleracao_resultante(self):
        forcas_internas = sum(self.forcas)
        forcas_externas = sum(self.forcas_externas.values)
        forca_resultante = forcas_internas + forcas_externas
        aceleracao_resultante = forca_resultante / self.massa
        return aceleracao_resultante

    def atualiza_estado(self, dt):
        a = self.aceleracao_resultante()
        self.velocidade += a * dt
        self.posicao += self.velocidade * dt

class Circulo(Objeto):
    def __init__(self, x, y, vx, vy, raio, massa=None, cor="roxo"):
        if massa is None:
            massa = raio ** 2
        super().__init__(x, y, vx, vy, massa)
        self.raio = grandezas.Posicao(raio)

    def centro(self):
        return self.posicao
    
    def plotar(self, canvas_id):
        pass
        # desenho.desenhar_arco(
        #     self.posicao.x, 
        #     self.posicao.y, 
        #     self.raio, 
        #     0, 
        #     2 * math.pi, 
        #     canvas_id, 
        #     self.cor
        # )
    
if __name__ == "__main__":
    from forcas import ForcaVetorial
    sim = Simulacao()

    cir = Circulo(5, 15, 5, 0, 1)
    cir.forcas.append(ForcaVetorial(0, -9.8))
    sim.adicionar_objeto(cir)
    sim.simular()


from js import p5

def setup():
    canvas = p5.createCanvas(400, 200)
    canvas.parent("canvas-container")  # Set the parent of the canvas
    p5.background(100)

def draw():
    p5.fill(255, 0, 0)
    p5.ellipse(p5.mouseX, p5.mouseY, 50, 50)

p5.setup = setup
p5.draw = draw


from js import document
import math
from .. import constantes
from .. import desenho

class Tartaruga:
    def __init__(
            self, 
            id_canvas_auxiliar=constantes.NOME_PAINEL_AUXILIAR,
            id_canvas_frente=constantes.NOME_PAINEL_FRENTE, 
            largura_canvas=constantes.LARGURA_PADRAO_CANVAS, 
            altura_canvas=constantes.ALTURA_PADRAO_CANVAS, 
            padrao_linha="solido",
            cor_linha="preto",
            peso_linha=1,
            cor_tartaruga="verde",
            tamanho_tartaruga=10,
        ):
        
        self.id_canvas_auxiliar = id_canvas_auxiliar
        self.id_canvas_frente = id_canvas_frente
        self.x = largura_canvas / 2
        self.y = altura_canvas / 2
        self.theta = 0  # Angulo inicial olhando para a direita
        self.caneta_abaixada = False
        self.largura_canvas = largura_canvas
        self.altura_canvas = altura_canvas

        self.mudar_cor_linha(cor_linha)
        self.mudar_padrao_linha(padrao_linha, peso_linha)
        
        self.tamanho_tartaruga = tamanho_tartaruga
        self.mudar_cor_tartaruga(cor_tartaruga)

    def abaixar_caneta(self):
        self.caneta_abaixada = True

    def levantar_caneta(self):
        self.caneta_abaixada = False

    def virar(self, angulo):
        self.theta += angulo
        self.desenhar_tartaruga()

    def andar(self, distancia):
        x_novo = self.x + math.cos(math.radians(self.theta)) * distancia
        y_novo = self.y - math.sin(math.radians(self.theta)) * distancia
        
        desenho.apagar_painel(self.id_canvas_frente)
        if self.caneta_abaixada:
            desenho.desenhar_linha(
                inicio_x=self.x, 
                inicio_y=self.y, 
                fim_x=x_novo, 
                fim_y=y_novo,
                id_canvas=self.id_canvas_auxiliar,
                cor=self.cor_linha_traduzida,
                largura=1,
                padrao=self.padrao_linha_traduzido,
            )
        
        self.x = x_novo
        self.y = y_novo

        self.desenhar_tartaruga()

    def desenhar_tartaruga(self):
        desenho.apagar_painel(self.id_canvas_frente)
        desenho.desenhar_triangulo(
            x_baricentro=self.x, 
            y_baricentro=self.y, 
            raio_circunscrito=self.tamanho_tartaruga, 
            cor=self.cor_tartaruga_traduzida, 
            id_canvas=self.id_canvas_frente, 
            angulo=self.theta
        )

    def voltar_para_casa(self):
        self.x = self.largura_canvas / 2
        self.y = self.altura_canvas / 2
        self.theta = 0
        desenho.apagar_painel(self.id_canvas_auxiliar)
        self.desenhar_tartaruga()

    def desenhar_linha(self, inicio_x, inicio_y, fim_x, fim_y):
        canvas = document.getElementById(self.id_canvas)
        ctx = canvas.getContext('2d')
        ctx.beginPath()
        ctx.moveTo(inicio_x, inicio_y)
        ctx.lineTo(fim_x, fim_y)
        ctx.stroke()

    def mudar_cor_linha(self, cor):
        self.cor_linha = cor
        cor_traduzida = constantes.traduzir(cor, constantes.TRADUCAO_CORES)
        self.cor_linha_traduzida = cor_traduzida

    def mudar_padrao_linha(self, padrao=None, peso_linha=None):
        if padrao:
            self.padrao_linha = padrao
            padrao_traduzido = constantes.traduzir(padrao, constantes.TRADUCAO_PADRAO_DE_LINHA)
            self.padrao_linha_traduzido = padrao_traduzido
        if peso_linha:
            self.peso_linha = peso_linha

    def mudar_tamanho_tartaruga(self, tamanho):
        self.tamanho_tartaruga = tamanho
        self.desenhar_tartaruga()

    def mudar_cor_tartaruga(self, cor):
        self.cor_tartaruga = cor
        cor_traduzida = constantes.traduzir(cor, constantes.TRADUCAO_CORES)
        self.cor_tartaruga_traduzida = cor_traduzida
        self.desenhar_tartaruga()




# if __name__ == "__main__":
        
    # Exemplo de uso:
    # # Supondo que as constantes de largura e altura do canvas sejam definidas em outro lugar do código.
    # largura_canvas = 500  # Exemplo de largura
    # altura_canvas = 300   # Exemplo de altura
    # tartaruga = Tartaruga(constantes.NOME_PAINEL_AUXILIAR, largura_canvas, altura_canvas)
    # tartaruga.abaixar_caneta()
    # tartaruga.andar(100)
    # tartaruga.virar(90)
    # tartaruga.andar(50)



import numpy as np

dicionario_emojis = {
    # coracoes
    ('coracao', 'azul'): '💙',  # U+1F499
    ('coracao', 'amarelo'): '💛',  # U+1F49B
    ('coracao', 'verde'): '💚',  # U+1F49A
    ('coracao', 'roxo'): '💜',  # U+1F49C

    # quadrados
    ('quadrado', 'azul'): '🟦',  # U+1F7E6
    ('quadrado', 'amarelo'): '🟨',  # U+1F7E8
    ('quadrado', 'verde'): '🟩',  # U+1F7E9
    ('quadrado', 'roxo'): '🟪',  # U+1F7EA

    # circulos
    ('circulo', 'azul'): '🔵',  # U+1F535
    ('circulo', 'amarelo'): '🟡',  # U+1F7E1
    ('circulo', 'verde'): '🟢',  # U+1F7E2
    ('circulo', 'roxo'): '🟣',  # U+1F7E3
}


def tabela_emojis(largura=12, altura=8):
    emojis_idx = np.random.choice(len(dicionario_emojis), size=(altura,largura), replace=True).tolist()
    idx = list(dicionario_emojis.keys())
    
    emojis_descricao = [[idx[shape_id] for shape_id in row] for row in emojis_idx]
    emojis = "\n".join(
        [
            "".join([dicionario_emojis[emo_desc] for emo_desc in row]) 
            for row in emojis_descricao
        ]
    )

    return emojis, emojis_descricao

# Example of accessing an emoji
emojis, emojis_descricao = (tabela_emojis())

print(emojis)
